# This software is licensed by the MIT License (MIT), see LICENSE file
#               Copyright © 2024 Gregory Lirent <opensource@lirent.ru>

from typing import Tuple


class OGClass(object):

    def check_fields(self):
        for k in self.props:
            if not hasattr(self.cls, k):
                raise AttributeError(f'type object {self.cls.__name__} has no attribute {k}')

    def get_metadata(self, obj, prefix=''):
        ctx = f'{prefix}{obj.og_name()}'
        for k in obj.og_properties():
            name = k.split('!', maxsplit=1)[0]
            name = f'{ctx}:{name}' if name else ctx
            v = getattr(obj, k.replace('!', ''))
            if not v:
                continue
            for x in v.all() if hasattr(v, 'all') else (v,):
                if hasattr(x, 'og_name') and x.og_name():
                    for _k, _v in self.get_metadata(x, ctx if x.og_name().startswith(':') else ''):
                        yield from obj.og_processing(_k, _v)
                else:
                    yield from obj.og_processing(name, str(x))

    def __init__(self, ctx: str = '', props: Tuple[str, ...] = ()):
        self.ctx = ctx.lower() if ctx else ''
        self.props = props

    def __call__(self, cls):
        self.cls = cls
        self.check_fields()

        if not hasattr(cls, 'og_processing'):
            cls.og_processing = classmethod(lambda ctx, k, v: (yield k, str(v)))

        cls.og_name = classmethod(lambda ctx: self.ctx)
        cls.og_properties = classmethod(lambda ctx: self.props)
        cls.og_metadata = lambda obj: (yield from self.get_metadata(obj))
        cls.og_metatags = lambda ctx: (f'<meta property="{k}" content="{v}" />' for k, v in ctx.og_metadata())

        return cls


class ProxyProperties(object):

    def __init__(self, *args):
        self.props = {}
        ctx = ''
        for arg in args:
            if isinstance(arg, str):
                ctx = arg
            else:
                self.props[ctx] = arg
        self.props.pop('', None)

    def __call__(self, cls):
        for k, props in self.props.items():
            ctx = cls.__dict__[k]
            for prop in props:
                setattr(cls, prop, property(
                    lambda obj: getattr(ctx, prop),
                    lambda obj, v: setattr(ctx, prop, v),
                    lambda obj: delattr(ctx, prop)
                ))
        return cls
