# This software is licensed by the MIT License (MIT), see LICENSE file
#               Copyright © 2024 Gregory Lirent <opensource@lirent.ru>

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.db import models

from . import decorators
from .page import OGSubpage
from .profile import OGProfile
from .properties import OGAudioStatic


class OGMusic(OGSubpage):
    """
    Namespace URI: https://ogp.me/ns/music#
    _________________________________________

    Types:

    music.radio_station
    music.playlist
    music.album
    music.song

    Entities:

    music:creator      -> The creators of the object.
    music:musician     -> The musicians that made the object.
    music:release_date -> The date the object was released.
    music:album        -> The album this object is from.
    music:album:disk   -> Which disc of the album this object is on.
    music:album:track  -> Which track this object is.
    music:song         -> The song on the object.
    music:song:disk    -> The same as music:album:disc but in reverse.
    music:song:track   -> The same as music:album:track but in reverse.
    music:duration     -> The object's length in seconds.

    _________________________________________
    """

    class Meta:
        abstract = True

    audio = models.ForeignKey(OGAudioStatic, null=True, on_delete=models.SET_NULL)

    @classmethod
    def create_song(cls, *args, **kwargs):
        return OGMusicSongPage(*args, **kwargs)

    @classmethod
    def create_album(cls, *args, **kwargs):
        return OGMusicAlbumPage(*args, **kwargs)

    @classmethod
    def create_playlist(cls, *args, **kwargs):
        return OGMusicPlaylistPage(*args, **kwargs)

    @classmethod
    def create_radio_station(cls, *args, **kwargs):
        return OGMusicRadioStationPage(*args, **kwargs)

    def __init__(self, *args, **kwargs):
        if self.__class__ not in (OGMusicSongPage, OGMusicAlbumPage, OGMusicPlaylistPage, OGMusicRadioStationPage):
            raise AttributeError('object creation is not allowed, '
                                 f'use {__class__.__name__}.create_[song | album | playlist | radio_station] instead')
        super().__init__(*args, **kwargs)


@decorators.OGClass(':song', ('!url', 'disc', 'track'))
class OGMusicSong(OGMusic):
    class Meta:
        db_table = 'music_songs'
        app_label = settings.get('OPENGRAPH_META_APP_LABEL', 'opengraph')
        verbose_name = _('music song meta information')
        verbose_name_plural = _('music songs meta information')

    disc = models.PositiveSmallIntegerField(_('disc number'), null=True)
    track = models.PositiveSmallIntegerField(_('track number'), null=True)

    def __init__(self, *args, **kwargs):
        kwargs['page_type'] = 'music.song'
        super().__init__(*args, **kwargs)


@decorators.OGClass('music', ('page', 'audio', 'disc', 'track', 'album!_url', 'duration', 'musician!s'))
class OGMusicSongPage(OGMusicSong):

    class Meta:
        proxy = True

    album = models.ForeignKey('OGMusicAlbumPage', on_delete=models.SET_NULL, null=True, verbose_name=_('album'))
    musicians = models.ManyToManyField(OGProfile, null=True, verbose_name=_('musicians'))
    duration = models.PositiveSmallIntegerField(_('duration'), null=True)

    @property
    def album_url(self) -> str | None:
        return self.album.url if self.album else None

    @classmethod
    def og_processing(cls, k, v):
        if k.endswith(':disc') or k.endswith(':track'):
            k = k.split(':')
            k.insert(1, 'album')
            k = ':'.join(k)
        yield k, v

    def __init__(self, *args, **kwargs):
        kwargs['page_type'] = 'music.song'
        super().__init__(*args, **kwargs)


class OGMusicPage(OGMusic):

    class Meta:
        db_table = 'music'
        app_label = settings.get('OPENGRAPH_META_APP_LABEL', 'opengraph')
        verbose_name = _('music object meta information')
        verbose_name_plural = _('music objects meta information')

    release_date = models.DateField(_('date of the release'), null=True)
    creators = models.ManyToManyField(OGProfile, null=True, verbose_name=_('creators'))
    songs = models.ManyToManyField(OGMusicSong, null=True, verbose_name=_('songs'))


@decorators.OGClass('music', ('page', 'audio', 'song!s', 'creator!s', 'release_date'))
class OGMusicAlbumPage(OGMusicPage):

    class Meta:
        proxy = True

    @classmethod
    def og_processing(cls, k, v, obj):
        if k.endswith(':creator'):
            k = f'{k[:-7]}musician'
        yield k, str(v)

    def __init__(self, *args, **kwargs):
        kwargs['page_type'] = 'music.album'
        super().__init__(*args, **kwargs)


@decorators.OGClass('music', ('page', 'audio', 'song!s', 'creator!s'))
class OGMusicPlaylistPage(OGMusicPage):

    class Meta:
        proxy = True

    def __init__(self, *args, **kwargs):
        kwargs['page_type'] = 'music.playlist'
        super().__init__(*args, **kwargs)


@decorators.OGClass('music', ('page', 'audio', 'creator!s'))
class OGMusicRadioStationPage(OGMusicPage):

    class Meta:
        proxy = True

    def __init__(self, *args, **kwargs):
        kwargs['page_type'] = 'music.radio_station'
        super().__init__(*args, **kwargs)
