# This software is licensed by the MIT License (MIT), see LICENSE file
#               Copyright © 2024 Gregory Lirent <opensource@lirent.ru>

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.db import models

from .. import decorators
from ..page import OGSubpage
from .properties import taxonomy, OGPrice


@decorators.OGClass('og:price', ('price', 'amount'))
class OGGlobalPrice(OGPrice):

    class Meta:
        proxy = True


@decorators.OGClass('product', ('price', 'brand', 'availability', 'condition', 'gender', 'gtin', 'material', 'locale'))
@decorators.ProxyProperties('page', 'locale')
class OGProductPage(OGSubpage):
    """
    Information URI: https://developers.facebook.com/docs/marketing-api/catalog/reference/#og-tags
    _________________________________________

    Entities: ->Catalog

    og:price                   -> Current price of the item.
    product:brand              -> Brand name of the item.
    product:availability       -> Current availability of the item:
                                  * in stock
                                  * out of stock
                                  * available for order
                                  * discontinued
    product:condition          -> Current condition of the item: new, refurbished, used.
    product:gender             -> Determines gender for sizing:
                                  * female
                                  * male
                                  * unisex.
    product:gtin               -> 14 DIG Product's Global Trade Item Number (GTINs). Exclude dashes and spaces.
    material                   -> Material the item is made from. Supported values:
                                  * cotton
                                  * denim
                                  * leather
    product:locale             -> Specifies which website version the product is from; for example, en_GB for the UK website.
    product:category           -> Represents the category of the product according to the Google's product taxonomy
                                  (https://www.google.com/basepages/producttype/taxonomy-with-ids.en-US.txt)
    product:price              ->
    product:mfr_part_no        ->
    product:catalog_id         ->
    product:mfr_part_no        ->
    product:sale_price         ->
    product:retailer_item_id   ->
    product:custom_label_[0-4] ->
    product:item_group_id      ->  see: Information URI


    _________________________________________
    """

    Categories = taxonomy.Categories

    class Materials(models.IntegerChoices):
        cotton = 1, _("cotton")
        denim = 2, _("denim")
        leather = 3, _("leather")

    class Conditions(models.IntegerChoices):
        new = 1, _("new")
        refurbished = 2, _("refurbished")
        used = 3, _("used")

    class Availabilities(models.IntegerChoices):
        in_stock = 1, _("in stock")
        out_of_stock = 2, _("out of stock")
        available = 3, _("available for order")
        discounted = 4, _("discontinued")

    class Genders(models.IntegerChoices):
        male = 1, _("male")
        female = 2, _("female")
        unisex = 3, _("unisex")

    class Meta:
        db_table = 'products'
        app_label = settings.get('OPENGRAPH_META_APP_LABEL', 'opengraph')
        verbose_name = _('product meta information')
        verbose_name_plural = _('products meta information')

    brand = models.CharField(_('brand'), max_length=64, null=True)
    gtin = models.CharField(_('gtin'), max_length=13, null=True)
    price = models.ForeignKey(OGGlobalPrice, on_delete=models.SET_NULL, verbose_name=_('price'), null=True)
    availability = models.PositiveSmallIntegerField(_('availability'), choices=Availabilities, null=True)
    condition = models.PositiveSmallIntegerField(_('condition'), choices=Conditions, null=True)
    material = models.PositiveSmallIntegerField(_('material'), choices=Materials, null=True)
    gender = models.PositiveSmallIntegerField(_('male'), choices=Genders.choices, null=True)
    category = models.PositiveIntegerField(_('category'), choices=Categories.choices, null=True)

    @classmethod
    def og_processing(cls, k, v, obj):
        if k.endswith((':availability', ':condition', ':material', ':gender')):
            if k.endswith(':material'):
                k = 'material'
                choices = cls.Materials
            elif k.endswith(':gender'):
                choices = cls.Genders
            else:
                choices = cls.Availabilities if k.endswith(':availability') else cls.Conditions
            v = choices(v).name.replace('_', ' ')
        if k.endswith(':category'):
            v = taxonomy.Categories.find_category(v)
        yield k, str(v)

    def __init__(self, *args, **kwargs):
        kwargs['page_type'] = 'product'
        super().__init__(*args, **kwargs)
