# This software is licensed by the MIT License (MIT), see LICENSE file
#               Copyright © 2024 Gregory Lirent <opensource@lirent.ru>

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.db import models

from . import decorators
from .page import OGSubpage


@decorators.OGClass
class OGProfile(OGSubpage):
    """
    Namespace URI: https://ogp.me/ns/profile#
    _________________________________________

    Entities:

    profile:first_name -> A name normally given to an individual by a parent or self-chosen.
    profile:last_name  -> A name inherited from a family or marriage and by which the individual is commonly known.
    profile:username   -> A short unique string to identify them.
    profile:gender     -> Their gender.

    _________________________________________
    """

    class Meta:
        db_table = 'profiles'
        app_label = settings.get('OPENGRAPH_META_APP_LABEL', 'opengraph')
        verbose_name = _('profile meta information')
        verbose_name_plural = _('profiles meta information')

    is_male = models.BooleanField(_('male'), null=True)
    first_name = models.CharField(_('first Name'), max_length=127, null=True)
    last_name = models.CharField(_('last Name'), max_length=127, null=True)
    username = models.CharField(_('username'), max_length=127, null=True)

    @property
    def gender(self) -> str | None:
        return None if self.is_male is None else 'male' if self.is_male else 'female'

    @gender.setter
    def gender(self, value) -> None:
        self.is_male = None if value not in ('male', 'female') else value == 'male'

    @gender.deleter
    def gender(self) -> None:
        self.is_male = None

    def __init__(self, *args, **kwargs):
        g = kwargs.pop('gender', '') or ''
        if g.lower() in ('male', 'female') and kwargs.get('is_male', None) is not None:
            kwargs['is_male'] = g.lower() == 'male'

        kwargs['page_type'] = 'profile'
        super().__init__(*args, **kwargs)


@decorators.OGClass('profile', ('page', 'username', 'first_name', 'last_name', 'gender'))
class OGProfilePage(OGProfile):

    class Meta:
        proxy = True
