# This software is licensed by the MIT License (MIT), see LICENSE file
#               Copyright © 2024 Gregory Lirent <opensource@lirent.ru>

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.db import models

from ... import decorators


@decorators.OGClass(':app', ('name', 'id', 'url'))
class TwitterCardAppInfo(models.Model):
    """
    URI: https://developers.facebook.com/blog/post/2013/06/05/platform-updates--new-open-graph-business-object-type/
    _________________________________________

    Entities:

    ...:app:name:... -> Name of The app
    ...:app:id:...   -> The app ID in the Google Play Store
    ...:app:url:...  -> The app's custom URL scheme

    _________________________________________
    """

    class Devices(models.IntegerChoices):
        IPHONE = 1, 'iphone'
        IPAD = 2, 'ipad'
        GOOGLEPLAY = 3, 'googleplay'

    class Meta:
        db_table = 'app'
        app_label = settings.get('TWITTER_CARD_META_APP_LABEL', 'twitter_card')
        verbose_name = _('app meta information')
        verbose_name_plural = _('apps meta information')

    type = models.PositiveSmallIntegerField(_('type'), choices=Devices.choices)
    name = models.CharField(_('name'), max_length=127)
    id = models.CharField(_('id'), max_length=64)
    url = models.URLField(_('url'), unique=True)

    @classmethod
    def og_processing(cls, k, v, obj):
        yield f'{k}:{cls.Devices(obj.type).name.lower()}', str(v)

    @classmethod
    def create_iphone_app(cls, *args, **kwargs):
        return TwitterCardIPhoneApp(*args, **kwargs)

    @classmethod
    def create_ipad_app(cls, *args, **kwargs):
        return TwitterCardIPadApp(*args, **kwargs)

    @classmethod
    def create_android_app(cls, *args, **kwargs):
        return TwitterCardAndroidApp(*args, **kwargs)

    def __init__(self, *args, **kwargs):
        if self.__class__ not in (TwitterCardIPhoneApp, TwitterCardIPadApp, TwitterCardAndroidApp):
            raise AttributeError('object creation is not allowed, '
                                 f'use {__class__.__name__}.create_[iphone | ipad | android]_app instead')
        super().__init__(*args, **kwargs)


class TwitterCardIPhoneApp(TwitterCardAppInfo):

    class Model:
        proxy = True

    def __init__(self, *args, **kwargs):
        kwargs['type'] = self.Device.IPHONE
        super().__init__(*args, **kwargs)


class TwitterCardIPadApp(TwitterCardAppInfo):

    class Model:
        proxy = True

    def __init__(self, *args, **kwargs):
        kwargs['type'] = self.Device.IPAD
        super().__init__(*args, **kwargs)


class TwitterCardAndroidApp(TwitterCardAppInfo):

    class Model:
        proxy = True

    def __init__(self, *args, **kwargs):
        kwargs['type'] = self.Device.ANDROID
        super().__init__(*args, **kwargs)
