# This software is licensed by the MIT License (MIT), see LICENSE file
#               Copyright © 2024 Gregory Lirent <opensource@lirent.ru>

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.db import models

from . import decorators
from .page import OGSubpage
from .profile import OGProfile
from .properties import OGVideoStatic, OGActor


class OGVideo(OGSubpage):
    class Meta:
        abstract = True

    video = models.ForeignKey(OGVideoStatic, null=True, on_delete=models.SET_NULL, verbose_name=_('video'))

    @classmethod
    def create_movie(cls, *args, **kwargs):
        return OGVideoMoviePage(*args, **kwargs)

    @classmethod
    def create_episode(cls, *args, **kwargs):
        return OGVideoEpisodePage(*args, **kwargs)

    @classmethod
    def create_tv_show(cls, *args, **kwargs):
        return OGVideoTVShowPage(*args, **kwargs)

    @classmethod
    def create_other(cls, *args, **kwargs):
        return OGVideoOtherPage(*args, **kwargs)

    def __init__(self, *args, **kwargs):
        if self.__class__ not in (OGVideoMoviePage, OGVideoEpisodePage, OGVideoTVShowPage, OGVideoOtherPage):
            raise AttributeError('object creation is not allowed, '
                                 f'use {__class__.__name__}.create_[movie | episode | tv_show | other] instead')
        super().__init__(*args, **kwargs)


class OGVideoPage(OGVideo):
    """
    Namespace URI: https://ogp.me/ns/music#
    _________________________________________

    Types:

    video.movie
    video.episode
    video.tv_show
    video.other

    Entities:

    video:actor        -> Actors in the video.
    video:actor:role   -> The role they played.
    video:director     -> Directors of the video.
    video:writer       -> Writers of the video.
    video:duration     -> The video's length in seconds.
    video:release_date -> The date the video was released.
    video:tag          -> Tag words associated with this video.
    video:series       -> Which series this video belongs to.

    _________________________________________
    """

    class Meta:
        db_table = 'video'
        app_label = settings.get('OPENGRAPH_META_APP_LABEL', 'opengraph')
        verbose_name = _('video object meta information')
        verbose_name_plural = _('video objects meta information')

    duration = models.PositiveSmallIntegerField(_('duration'), null=True)
    release_date = models.DateField(_('date of the release'), null=True)
    actors = models.ManyToManyField(OGActor, null=True, verbose_name=_('actors'))
    directors = models.ManyToManyField(OGProfile, null=True, verbose_name=_('directors'))
    writers = models.ManyToManyField(OGProfile, null=True, verbose_name=_('writers'))
    series = models.ForeignKey('OGVideoPage', on_delete=models.CASCADE)


@decorators.OGClass('video', ('page', 'video', 'actors', 'director!s', 'writer!s', 'release_date', 'duration'))
class OGVideoMoviePage(OGVideoPage):

    class Meta:
        proxy = True

    def __init__(self, *args, **kwargs):
        kwargs['page_type'] = f'video.movie'
        super().__init__(*args, **kwargs)


@decorators.OGClass('video', ('page', 'video', 'actor!s', 'director!s', 'writer!s', 'release_date', 'duration'))
class OGVideoTVShowPage(OGVideoPage):

    class Meta:
        proxy = True

    def __init__(self, *args, **kwargs):
        kwargs['page_type'] = f'video.tv_show'
        super().__init__(*args, **kwargs)


@decorators.OGClass('video', ('page', 'video', 'actor!s', 'director!s', 'writer!s', 'release_date', 'duration'))
class OGVideoOtherPage(OGVideoPage):
    class Meta:
        proxy = True

    def __init__(self, *args, **kwargs):
        kwargs['page_type'] = f'video.other'
        super().__init__(*args, **kwargs)


@decorators.OGClass('video', ('page', 'video', 'actor!s', 'director!s', 'writer!s', 'release_date', 'duration', 'series'))
class OGVideoEpisodePage(OGVideoPage):

    class Meta:
        proxy = True

    def __init__(self, *args, **kwargs):
        kwargs['page_type'] = f'video.episode'
        super().__init__(*args, **kwargs)
